#include <X11/Xlib.h>
#include <stdio.h>
#include <stdlib.h> 
#include <string.h>
#include <math.h>


// ====================================
Display *Mydisplay;
Window  Mywindow;
XEvent  Myevent;
int     Myscreen;
GC	MyGC;
int WinW,WinH;
int SDepth;
Drawable Mydrawable;
Pixmap Mypixmap;
int MyOldCol=-1;

void MyInit(int winwidth,int winheight){
 Mydisplay=XOpenDisplay(NULL);
 // Open connection to server
 if(Mydisplay==NULL){
  printf("couldn't open display\n");
  exit(1);
 }
 Myscreen=DefaultScreen(Mydisplay);
 // Create window
 Mywindow=XCreateSimpleWindow(Mydisplay, RootWindow(Mydisplay, Myscreen), 10,10,winwidth,winheight, 1,
                            WhitePixel(Mydisplay,Myscreen),BlackPixel(Mydisplay,Myscreen) );
 // Process Window Close Event through event handler so XNextEvent does not fail
 // note: look up these functions to try to understand how/why this works
 Atom atom = XInternAtom( Mydisplay, "WM_DELETE_WINDOW", 0 );
 XSetWMProtocols(Mydisplay, Mywindow, &atom, 1);
 atom = XInternAtom( Mydisplay, "PRIMARY", 0 );
 // Select the kind of events we're interested in 
 XSelectInput(Mydisplay, Mywindow, ExposureMask | KeyPressMask | KeyReleaseMask | ButtonPressMask | ButtonReleaseMask | StructureNotifyMask );
 // Map (show) the window
 XMapWindow(Mydisplay, Mywindow);
 // test
 MyGC = DefaultGC(Mydisplay,Myscreen);
 XFontStruct *xfontstruct = XLoadQueryFont(Mydisplay,"-*-*-medium-r-normal-*-17-120-100-100-*-0-*-*");
 XSetFont(Mydisplay, MyGC, xfontstruct->fid);

 XSetForeground(Mydisplay,MyGC,WhitePixel(Mydisplay,Myscreen));
 XSetBackground(Mydisplay,MyGC,BlackPixel(Mydisplay,Myscreen));

 //XDrawLine(Mydisplay, Mywindow, MyGC, 0,0, winwidth,winheight);

 XStoreName(Mydisplay,Mywindow, "Test");

 WinW=winwidth;
 WinH=winheight;

 printf("whitepixel %d\n",WhitePixel(Mydisplay,Myscreen));
 SDepth=0;
 int i,wp;
 wp = WhitePixel(Mydisplay,Myscreen);
 for(i=0;i<31;i++){
  SDepth += (wp & 1);
  wp = wp >> 1;
 }
 printf("SDepth %d\n",SDepth);

 Mypixmap = XCreatePixmap(Mydisplay,Mywindow,WinW,WinH,SDepth);
 Mydrawable = Mypixmap;

}

void MyCleanup(){
 XDestroyWindow(Mydisplay, Mywindow);
 XCloseDisplay(Mydisplay);
}

// ===========================================================

void Cls(){
 //XClearWindow(Mydisplay,Mydrawable);
 //XClearArea(Mydisplay,Mydrawable, 0,0,WinW,WinH, 0);
#if 1
 XSetForeground(Mydisplay, MyGC,0);//BlackPixel(Mydisplay,Myscreen) );
#else
 XSetForeground(Mydisplay, MyGC,127);
#endif
 XFillRectangle(Mydisplay,Mydrawable,MyGC, 0,0, WinW,WinH);
 XSetForeground(Mydisplay, MyGC,MyOldCol );
}

int MyColour(int r, int g, int b){
 r=r&255;
 g=g&255;
 b=b&255;
 int o;
 switch(SDepth){
 case 24:
  o = ((r&255)<<16) | ((g&255)<<8) | (b&255);
  break;
 case 16:
  o = ((r>>3)<<11) | ((g>>3)<<6) | (b>>3);
  break;
 case 15:
  o = ((r>>3)<<10) | ((g>>3)<<5) | (b>>3);
  break;
 default:
  o = (r|g|b) > 127;
 }
 MyOldCol=o;
 return o;
}

void Gcol(int r, int g, int b){
 int c = MyColour(r,g,b);
 XSetForeground(Mydisplay,MyGC,c);
}

void GcolBG(int r, int g, int b){
 int c = MyColour(r,g,b);
 XSetBackground(Mydisplay,MyGC,c);
}

void GcolGrey(int g){
 int c = MyColour(g,g,g);
 XSetForeground(Mydisplay,MyGC,c);
}

void GcolDirect(int rgb){
 XSetForeground(Mydisplay,MyGC,rgb);
}

void MySetWindowBackground(int r, int g, int b){
 int c = MyColour(r,g,b);
 XSetWindowBackground(Mydisplay,Mydrawable,c);
}

void Rectangle(int x,int y,int w,int h){
 XDrawRectangle(Mydisplay,Mydrawable,MyGC, x,y, w,h);
}

void RectangleFill(int x,int y,int w,int h){
 XFillRectangle(Mydisplay,Mydrawable,MyGC, x,y, w,h);
}

void Line(int x,int y,int xb,int yb){
 XDrawLine(Mydisplay, Mydrawable, MyGC, x,y, xb,yb);
}

void Circle(int x,int y, int r){
 XDrawArc(Mydisplay, Mydrawable, MyGC, x-r/2, y-r/2, r, r, 0,321*64*2 );
}

void CircleFill(int x,int y, int r){
 XFillArc(Mydisplay, Mydrawable, MyGC, x-r/2, y-r/2, r, r, 0,321*64*2 );
}

// ===========================================================

void MyCopy(){
 XCopyArea(Mydisplay,Mypixmap,Mywindow,MyGC,0,0,WinW,WinH,0,0);
}

// ===========================================================

void MyHandleEvents_WithBlocking(){
int i;

 while(1){
  XNextEvent(Mydisplay, &Myevent);

  printf("event.type: %d\n",Myevent.type);



  switch(Myevent.type){
  case Expose:
   //PROCnuts();
   break;
  case ConfigureNotify:
   WinW = Myevent.xconfigure.width;
   WinH = Myevent.xconfigure.height;
   XFreePixmap(Mydisplay,Mypixmap);
   Mypixmap = XCreatePixmap(Mydisplay,Mywindow,WinW,WinH,SDepth);
   break;
  case ClientMessage:
   //system("Shia");
   printf("clientmessage exit\n");
   MyCleanup();
   exit(0);
   break;
  case ButtonPress:
   MyCopy();
   break;
  }

 }
 
}

void MyHandleEvents(){
 int i;

 while(1){
#if 0
  XNextEvent(Mydisplay, &Myevent);
#else 
  if( XCheckMaskEvent( Mydisplay,ExposureMask | KeyPressMask | ButtonPressMask | ButtonReleaseMask | StructureNotifyMask, &Myevent )==False ) break;
#endif
  printf("event.type: %d\n",Myevent.type);



  switch(Myevent.type){
  case Expose:
   Cls();
   //PROCnuts();
   break;
  case ConfigureNotify:
   WinW = Myevent.xconfigure.width;
   WinH = Myevent.xconfigure.height;
   usleep(5000);
   break;
  case ButtonPress:
   while( Myevent.type != ButtonRelease ) XNextEvent(Mydisplay, &Myevent);
   break;
  case ButtonRelease:
   //system("Shia");
   MyCleanup();
   exit(0);
   break;
  }

 }
 
}



int OLDRUBISHmain(void){
 MyInit(640,480);

 MyHandleEvents_WithBlocking(); 

 MyCleanup();
 return 0;
}